/*
 * I2C support for the X1600.
 *
 * Copyright (C) 2017, 2018, 2019, 2023 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <l4/sys/types.h>
#include <stdint.h>



#ifdef __cplusplus

#include <l4/devices/cpm-x1600.h>
#include <l4/devices/hw_mmio_register_block.h>

// I2C channel.

class I2c_x1600_channel
{
private:
  Hw::Register_block<32> _regs;
  Cpm_x1600_chip *_cpm;
  uint32_t _frequency;

  // Buffer management.

  unsigned int _pos, _reqpos, _total;
  uint8_t *_buf;

  // Status conditions.

  int _fail;
  int _stop;

public:
  I2c_x1600_channel(l4_addr_t start, enum Clock_identifiers clock,
                    Cpm_x1600_chip *cpm, uint32_t frequency);

  uint32_t get_frequency();
  void set_target(uint8_t addr);

  // Reading initiation and execution.

  void start_read(uint8_t buf[], unsigned int total, int stop = 0);
  void read();

  // Writing initiation and execution.

  void start_write(uint8_t buf[], unsigned int total, int stop = 0);
  void write();

  // Transaction control.

  void stop();
  void disable();

  // Specific status conditions.

  unsigned int have_read();
  unsigned int have_written();
  int read_done();
  int write_done();

  int failed();
  int read_failed();
  int write_failed();

private:
  void enable();

  int active();
  int have_input();
  int have_output();
  int can_send();

  void reset_flags();
  void init_parameters();
  void set_frequency();

  void set_read_threshold();
  void queue_reads();
  void queue_writes();
  void store_reads();
};

// I2C device control.

class I2c_x1600_chip
{
private:
  l4_addr_t _start, _end;
  Cpm_x1600_chip *_cpm;
  uint32_t _frequency;

public:
  I2c_x1600_chip(l4_addr_t start, l4_addr_t end, Cpm_x1600_chip *cpm,
                  uint32_t frequency);

  I2c_x1600_channel *get_channel(uint8_t channel);
};

#endif /* __cplusplus */



/* C language interface. */

EXTERN_C_BEGIN

void *x1600_i2c_init(l4_addr_t start, l4_addr_t end, void *cpm,
                      uint32_t frequency);

void *x1600_i2c_get_channel(void *i2c, uint8_t channel);

uint32_t x1600_i2c_get_frequency(void *i2c_channel);

void x1600_i2c_set_target(void *i2c_channel, uint8_t addr);

void x1600_i2c_start_read(void *i2c_channel, uint8_t buf[], unsigned int total,
                          int stop);

void x1600_i2c_read(void *i2c_channel);

void x1600_i2c_start_write(void *i2c_channel, uint8_t buf[], unsigned int total,
                           int stop);

void x1600_i2c_write(void *i2c_channel);

int x1600_i2c_read_done(void *i2c_channel);

int x1600_i2c_write_done(void *i2c_channel);

unsigned int x1600_i2c_have_read(void *i2c_channel);

unsigned int x1600_i2c_have_written(void *i2c_channel);

int x1600_i2c_failed(void *i2c_channel);

EXTERN_C_END
